@@ -1,7 +1,7 @@
 use hmac_sha512::sha384 as hmac_sha384;
 use rsa::{
     BigUint, PrivateKeyEncoding as _, PrivateKeyPemEncoding as _, PublicKey as _,
-    PublicKeyEncoding as _, PublicKeyPemEncoding as _,
+    PublicKeyEncoding as _, PublicKeyParts as _, PublicKeyPemEncoding as _,
 };
 use serde::{de::DeserializeOwned, Serialize};
 use std::convert::TryFrom;
@@ -22,6 +22,11 @@ impl AsRef<rsa::RSAPublicKey> for RSAPublicKey {
     }
 }
 
+pub struct RSAPublicKeyComponents {
+    pub n: Vec<u8>,
+    pub e: Vec<u8>,
+}
+
 impl RSAPublicKey {
     pub fn from_der(der: &[u8]) -> Result<Self, Error> {
         let rsa_pk = rsa::RSAPublicKey::from_pkcs8(der)?;
@@ -48,6 +53,12 @@ impl RSAPublicKey {
     pub fn to_pem(&self) -> Result<String, Error> {
         self.0.to_pem_pkcs8().map_err(Into::into)
     }
+
+    pub fn to_components(&self) -> RSAPublicKeyComponents {
+        let n = self.0.n().to_bytes_be();
+        let e = self.0.e().to_bytes_be();
+        RSAPublicKeyComponents { n, e }
+    }
 }
 
 #[doc(hidden)]
@@ -290,6 +301,10 @@ impl RS256PublicKey {
         self.pk.to_pem()
     }
 
+    pub fn to_components(&self) -> RSAPublicKeyComponents {
+        self.pk.to_components()
+    }
+
     pub fn with_key_id(mut self, key_id: &str) -> Self {
         self.key_id = Some(key_id.to_string());
         self
@@ -431,6 +446,10 @@ impl RS512PublicKey {
         self.pk.to_pem()
     }
 
+    pub fn to_components(&self) -> RSAPublicKeyComponents {
+        self.pk.to_components()
+    }
+
     pub fn with_key_id(mut self, key_id: &str) -> Self {
         self.key_id = Some(key_id.to_string());
         self
@@ -572,6 +591,10 @@ impl RS384PublicKey {
         self.pk.to_pem()
     }
 
+    pub fn to_components(&self) -> RSAPublicKeyComponents {
+        self.pk.to_components()
+    }
+
     pub fn with_key_id(mut self, key_id: &str) -> Self {
         self.key_id = Some(key_id.to_string());
         self
@@ -713,6 +736,10 @@ impl PS256PublicKey {
         self.pk.to_pem()
     }
 
+    pub fn to_components(&self) -> RSAPublicKeyComponents {
+        self.pk.to_components()
+    }
+
     pub fn with_key_id(mut self, key_id: &str) -> Self {
         self.key_id = Some(key_id.to_string());
         self
@@ -854,6 +881,10 @@ impl PS512PublicKey {
         self.pk.to_pem()
     }
 
+    pub fn to_components(&self) -> RSAPublicKeyComponents {
+        self.pk.to_components()
+    }
+
     pub fn with_key_id(mut self, key_id: &str) -> Self {
         self.key_id = Some(key_id.to_string());
         self
@@ -995,6 +1026,10 @@ impl PS384PublicKey {
         self.pk.to_pem()
     }
 
+    pub fn to_components(&self) -> RSAPublicKeyComponents {
+        self.pk.to_components()
+    }
+
     pub fn with_key_id(mut self, key_id: &str) -> Self {
         self.key_id = Some(key_id.to_string());
         self
